@@ -1,11 +1,12 @@
+use crate::opt::{ProcessCategory, SystemCategory};
 use crate::types::ProcessInfo;
 use crate::{CpuInfo, GpuInfo};
 use std::path::Path;
 
 pub fn consume<P: AsRef<Path>>(
     path: P,
-    categories: &[String],
-    sys_categories: &[String],
+    proc_categories: &[ProcessCategory],
+    sys_categories: &[SystemCategory],
     timestamps: &[chrono::DateTime<chrono::Local>],
     processes: &[ProcessInfo],
     cpu_info: &[CpuInfo],
@@ -17,14 +18,13 @@ pub fn consume<P: AsRef<Path>>(
         .unwrap();
 
     // Process
-    for (ci, c) in categories.into_iter().enumerate() {
+    for (ci, &c) in proc_categories.into_iter().enumerate() {
         // Title
-        wtr.write_field(match c.as_str() {
-            "cpu" => "Process CPU Usage",
-            "mem" => "Process Memory Usage",
-            "gpu" => "Process GPU Usage",
-            "fps" => "Process FPS",
-            _ => unimplemented!(),
+        wtr.write_field(match c {
+            ProcessCategory::CPU => "Process CPU Usage",
+            ProcessCategory::Mem => "Process Memory Usage",
+            ProcessCategory::GPU => "Process GPU Usage",
+            ProcessCategory::FPS => "Process FPS",
         })
         .unwrap();
         for p in processes {
@@ -47,9 +47,9 @@ pub fn consume<P: AsRef<Path>>(
     }
 
     // System
-    for c in sys_categories {
-        match c.as_str() {
-            "sys_cpu_freq" => {
+    for &c in sys_categories {
+        match c {
+            SystemCategory::CPUFreq => {
                 // Title
                 wtr.write_field("CPU Frequency").unwrap();
                 for i in 0..cpu_info.len() {
@@ -68,7 +68,7 @@ pub fn consume<P: AsRef<Path>>(
                     wtr.write_record(None::<&[u8]>).unwrap();
                 }
             }
-            "sys_gpu" => {
+            SystemCategory::GPU => {
                 // Title
                 wtr.write_field("System GPU Usage").unwrap();
                 for _ in 0..gpu_info.len() {
@@ -87,7 +87,6 @@ pub fn consume<P: AsRef<Path>>(
                     wtr.write_record(None::<&[u8]>).unwrap();
                 }
             }
-            _ => unimplemented!(),
         }
         wtr.write_record([" "]).unwrap();
     }
