@@ -1,9 +1,9 @@
-use crate::opt::Opts;
+use crate::opt::{Category, Opts, ProcessCategory, SystemCategory};
 use crate::types::{CpuInfo, GpuInfo};
 use clap::Parser;
 use precord_core::{Features, Pid, System};
-use std::thread;
 use std::time::{Duration, Instant};
+use std::{fs, thread};
 
 mod consumer_csv;
 mod consumer_json;
@@ -13,30 +13,49 @@ mod types;
 mod utils;
 
 fn main() {
-    let mut opts: Opts = Opts::parse();
-    let sys_category = utils::drain_filter_vec(&mut opts.category, |c| c.starts_with("sys_"));
+    let opts: Opts = Opts::parse();
+    let proc_category: Vec<_> = opts
+        .category
+        .iter()
+        .filter_map(|&c| match c {
+            Category::CPU => Some(ProcessCategory::CPU),
+            Category::Mem => Some(ProcessCategory::Mem),
+            Category::GPU => Some(ProcessCategory::GPU),
+            Category::FPS => Some(ProcessCategory::FPS),
+            _ => None,
+        })
+        .collect();
+    let sys_category: Vec<_> = opts
+        .category
+        .iter()
+        .flat_map(|&c| match c {
+            Category::SysCPUFreq => Some(SystemCategory::CPUFreq),
+            Category::SysGPU => Some(SystemCategory::GPU),
+            _ => None,
+        })
+        .collect();
 
     let mut timestamps = vec![];
 
     let (processes, cpu_info, cpu_frequency_max, gpu_info) = {
         let mut features = Features::PROCESS;
 
-        if opts.category.contains(&"gpu".to_owned())
-            || sys_category.contains(&"sys_gpu".to_string())
+        if proc_category.contains(&ProcessCategory::GPU)
+            || sys_category.contains(&SystemCategory::GPU)
         {
             features.insert(Features::GPU);
         }
-        if opts.category.contains(&"fps".to_string()) {
+        if proc_category.contains(&ProcessCategory::FPS) {
             features.insert(Features::FPS);
         }
-        if sys_category.contains(&"sys_cpu_freq".to_string()) {
+        if sys_category.contains(&SystemCategory::CPUFreq) {
             features.insert(Features::CPU_FREQUENCY);
         }
 
-        let mut system = System::new(features, []);
+        let mut system = System::new(Features::PROCESS, []);
         system.update();
 
-        let mut processes = opts.find_processes(&system);
+        let mut processes = opts.find_processes(&system, proc_category.len());
 
         let mut system = System::new(features, processes.iter().map(|p| p.pid));
         system.update();
@@ -64,9 +83,9 @@ fn main() {
             'p: for process in processes.iter_mut() {
                 let mut message = format!("{}({})", &process.name, process.pid);
 
-                for (idx, c) in opts.category.iter().enumerate() {
-                    match c.as_str() {
-                        "cpu" => {
+                for (idx, &c) in proc_category.iter().enumerate() {
+                    match c {
+                        ProcessCategory::CPU => {
                             if let Some(cpu_usage) = system.process_cpu_usage(process.pid) {
                                 process.values[idx].push(cpu_usage);
 
@@ -76,7 +95,7 @@ fn main() {
                                 continue 'p;
                             }
                         }
-                        "mem" => {
+                        ProcessCategory::Mem => {
                             if let Some(mem_usage) = system.process_mem(process.pid) {
                                 let mem_usage = mem_usage / 1024.0;
                                 process.values[idx].push(mem_usage);
@@ -87,7 +106,7 @@ fn main() {
                                 continue 'p;
                             }
                         }
-                        "gpu" => {
+                        ProcessCategory::GPU => {
                             if let Some(gpu_usage) = system.process_gpu_usage(process.pid) {
                                 process.values[idx].push(gpu_usage);
 
@@ -97,13 +116,12 @@ fn main() {
                                 continue 'p;
                             }
                         }
-                        "fps" => {
+                        ProcessCategory::FPS => {
                             let fps = system.process_fps(process.pid);
                             process.values[idx].push(fps);
 
                             message.push_str(&format!(" / FPS {}", fps));
                         }
-                        _ => unimplemented!(),
                     }
                 }
 
@@ -111,9 +129,9 @@ fn main() {
             }
 
             // System
-            for c in sys_category.iter() {
-                match c.as_str() {
-                    "sys_cpu_freq" => {
+            for &c in sys_category.iter() {
+                match c {
+                    SystemCategory::CPUFreq => {
                         let cpu_frequency = system.cpu_frequency();
 
                         println!(
@@ -140,7 +158,7 @@ fn main() {
                             }
                         }
                     }
-                    "sys_gpu" => {
+                    SystemCategory::GPU => {
                         let sys_gpu_usage = system.system_gpu_usage().unwrap();
 
                         println!("System GPU Usage: {}%", sys_gpu_usage);
@@ -153,7 +171,6 @@ fn main() {
                             gpu_info[0].usage.push(sys_gpu_usage);
                         }
                     }
-                    _ => unreachable!(),
                 }
             }
 
@@ -168,11 +185,17 @@ fn main() {
     };
 
     for output in opts.output.iter() {
+        if let Some(parent) = output.parent() {
+            if parent.components().count() > 0 && !parent.exists() {
+                fs::create_dir_all(parent).unwrap();
+            }
+        }
+
         if let Some(ext) = output.extension() {
             if ext == "csv" {
                 consumer_csv::consume(
                     output,
-                    &opts.category,
+                    &proc_category,
                     &sys_category,
                     &timestamps,
                     &processes,
@@ -183,6 +206,7 @@ fn main() {
                 consumer_svg::consume(
                     output,
                     &opts,
+                    &proc_category,
                     &sys_category,
                     &processes,
                     &cpu_info,
@@ -192,7 +216,7 @@ fn main() {
             } else if ext == "json" {
                 consumer_json::consume(
                     output,
-                    &opts.category,
+                    &proc_category,
                     &sys_category,
                     &timestamps,
                     &processes,
