@@ -10,7 +10,7 @@ pub struct ProcessInfo {
 }
 
 impl ProcessInfo {
-    pub fn new(system: &System, categories: &[String], pid: Pid) -> Self {
+    pub fn new(system: &System, proc_category_len: usize, pid: Pid) -> Self {
         let name = system
             .process_name(pid)
             .expect(&format!("No such process({})", pid))
@@ -24,7 +24,7 @@ impl ProcessInfo {
             pid,
             name,
             command,
-            values: vec![vec![]; categories.len()],
+            values: vec![vec![]; proc_category_len],
             valid: true,
         }
     }
